package exnihilocreatio.barrel;

import exnihilocreatio.networking.MessageBarrelModeUpdate;
import exnihilocreatio.networking.PacketHandler;
import exnihilocreatio.registries.registries.BarrelModeRegistry;
import exnihilocreatio.registries.registries.BarrelModeRegistry.TriggerType;
import exnihilocreatio.tiles.TileBarrel;
import net.minecraft.block.state.IBlockState;
import net.minecraft.item.ItemStack;
import net.minecraftforge.items.ItemStackHandler;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.List;

public class BarrelItemHandler extends ItemStackHandler {

    private TileBarrel barrel;

    public BarrelItemHandler(TileBarrel barrel) {
        super(1);
        this.barrel = barrel;
    }

    @Override
    protected int getStackLimit(int slot, @Nonnull ItemStack stack) {
        return 1;
    }

    @Override
    @Nonnull
    public ItemStack getStackInSlot(int slot) {
        if (barrel.getMode() != null && barrel.getMode().getHandler(barrel) != null) {
            return barrel.getMode().getHandler(barrel).getStackInSlot(slot);
        }

        return ItemStack.field_190927_a;
    }

    @Override
    public void setStackInSlot(int slot, @Nonnull ItemStack stack) {
        if (barrel.getMode() != null && barrel.getMode().isTriggerItemStack(stack)) {
            barrel.getMode().addItem(stack, barrel);
            barrel.func_70296_d();

            IBlockState state = barrel.func_145831_w().func_180495_p(barrel.func_174877_v());
            barrel.func_145831_w().func_175656_a(barrel.func_174877_v(), state);
        } else if (barrel.getMode() != null && barrel.getMode().getHandler(barrel) != null) {
            barrel.getMode().getHandler(barrel).setStackInSlot(slot, stack);
        } else if (barrel.getMode() == null) {
            List<IBarrelMode> modes = BarrelModeRegistry.getModes(TriggerType.ITEM);

            if (modes != null) {
                for (IBarrelMode possibleMode : modes) {
                    if (possibleMode.isTriggerItemStack(stack)) {
                        barrel.setMode(possibleMode.getName());
                        PacketHandler.sendToAllAround(new MessageBarrelModeUpdate(barrel.getMode().getName(), barrel.func_174877_v()), barrel);

                        barrel.getMode().addItem(stack, barrel);
                        barrel.func_70296_d();

                        IBlockState state = barrel.func_145831_w().func_180495_p(barrel.func_174877_v());
                        barrel.func_145831_w().func_175656_a(barrel.func_174877_v(), state);
                    }
                }
            }
        }
    }

    @Override
    @Nonnull
    public ItemStack extractItem(int slot, int amount, boolean simulate) {
        if (barrel.getMode() != null && barrel.getMode().getHandler(barrel) != null) {
            return barrel.getMode().getHandler(barrel).extractItem(slot, amount, simulate);
        }

        return ItemStack.field_190927_a;
    }

    @Override
    @Nonnull
    public ItemStack insertItem(int slot, @Nonnull ItemStack stack, boolean simulate) {
        if (barrel.getMode() == null) {
            ArrayList<IBarrelMode> modes = BarrelModeRegistry.getModes(TriggerType.ITEM);

            if (modes == null) {
                return stack;
            }

            for (IBarrelMode possibleMode : modes) {
                if (possibleMode.isTriggerItemStack(stack)) {
                    if (!simulate) {
                        barrel.setMode(possibleMode.getName());
                        PacketHandler.sendToAllAround(new MessageBarrelModeUpdate(barrel.getMode().getName(), barrel.func_174877_v()), barrel);
                        barrel.getMode().addItem(stack, barrel);
                        barrel.func_70296_d();
                        IBlockState state = barrel.func_145831_w().func_180495_p(barrel.func_174877_v());
                        barrel.func_145831_w().func_175656_a(barrel.func_174877_v(), state);
                    }

                    ItemStack ret = stack.func_77946_l();
                    ret.func_190918_g(1);
                    return ret;
                }
            }

            return stack;
        } else {
            if (barrel.getMode().getHandler(barrel) != null)
                return barrel.getMode().getHandler(barrel).insertItem(slot, stack, simulate);
        }

        return stack;
    }

}
